/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */



import antlr.RecognitionException;

/**
 *
 * @author ycao
 */
public class VinlDecimal implements VinlType {
	
	private double value;
	private String name;

	public VinlDecimal() {
		//auto initialize all decimal to 0 if no value is specified
		value = 0;
	}
	
	//use for tmp variables
	public VinlDecimal(double value) {
		this.value = value;
	}
	
	public VinlDecimal(String name) {
		this.name = name;
	}

	public VinlDecimal(String name, double value) {
		this.name = name;
		this.value = value;
	}
	
	
	public int getType() {
		return VinlType.DECIMAL;
	}

	public void setValue(double v) {
		this.value = v;
	}

	public double getValue() {
		return this.value;
	}

	public void setName(String name) {
		this.name=name;
	}

	public String getName() {
		return this.name;
	}

	public VinlType eval(int op, VinlType operand1, VinlType operand2) throws RecognitionException {

		if(!checkValid(operand1)) {
			throw new RecognitionException("First operand is not decimal or it has no value");
		}
		if(!checkValid(operand2)) {
			throw new RecognitionException("Second operand is not decimal or it has no value");
		}
		VinlDecimal op1 = (VinlDecimal)operand1;	
		VinlDecimal op2 = (VinlDecimal)operand2;	
		/* decimal supports all arithmetical and relational operaionts */
		switch(op) {
			case VinlOperation.PLUS: 
				return op1.plus(op2);
			case VinlOperation.MINUS: 
				return op1.minus(op2);
			case VinlOperation.MULTI: 
				return op1.multi(op2);
			case VinlOperation.DIV: 
				return op1.div(op2);
			case VinlOperation.MOD: 
				return op1.mod(op2);
			case VinlOperation.EQ: 
				return op1.eq(op2);
			case VinlOperation.NEQ: 
				return op1.neq(op2);
			case VinlOperation.LT: 
				return op1.lt(op2);
			case VinlOperation.GT: 
				return op1.gt(op2);
			case VinlOperation.LE: 
				return op1.le(op2);
			case VinlOperation.GE: 
				return op1.ge(op2);
			default:
				throw new antlr.RecognitionException("Decimal does not support this operation");
		}
	}

	public boolean checkValid(VinlType operand) {
		return (operand.getType()==VinlType.DECIMAL);
	}
	
	//arithmetic oeprations 
	public VinlDecimal plus(VinlDecimal b) {
		return new VinlDecimal(value+b.getValue());
	}

	public VinlDecimal minus(VinlDecimal b) {
		return new VinlDecimal(value-b.getValue());
	}
	
	public VinlDecimal multi(VinlDecimal b) {
		return new VinlDecimal(value*b.getValue());
	}

	public VinlDecimal div(VinlDecimal b) {
		return new VinlDecimal(value/b.getValue());
	}

	public VinlDecimal mod(VinlDecimal b) {
		return new VinlDecimal(value%b.getValue());
	}

	//relational operations
	public VinlBoolean eq(VinlDecimal b) {
		return new VinlBoolean(this.value == b.getValue());
	}

	public VinlBoolean neq(VinlDecimal b) {
		return new VinlBoolean(this.value != b.getValue());
	}

	public VinlBoolean lt(VinlDecimal b) {
		return new VinlBoolean(this.value < b.getValue());
	}

	public VinlBoolean le(VinlDecimal b) {
		return new VinlBoolean(this.value <= b.getValue());
	}

	public VinlBoolean gt(VinlDecimal b) {
		return new VinlBoolean(this.value > b.getValue());
	}

	public VinlBoolean ge(VinlDecimal b) {
		return new VinlBoolean(this.value >= b.getValue());
	}
}
