/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import antlr.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.ArrayList;

/**
 * This class is the main GUI class
 * @author ycao
 */
public class VinlInterpreterGui extends JFrame implements KeyListener {

	private static VinlGraphicPanel gp;
	private static JTextField editor;
	private ArrayList<String> cmdlist = new ArrayList();
	private int cmd_index = 0;

	public VinlInterpreterGui() {
		gp = new VinlGraphicPanel();
		gp.setPreferredSize(new Dimension(600, 600));
		editor = new JTextField();
		editor.setPreferredSize(new Dimension(500, 30));
		editor.addKeyListener(this);

		this.getContentPane().setLayout(new java.awt.BorderLayout(0, 10));
		this.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		this.getContentPane().add(gp, java.awt.BorderLayout.CENTER);
		this.getContentPane().add(editor, java.awt.BorderLayout.PAGE_END);
		this.setTitle("Interperter for VINL -- VINL is Not Logo");

		pack();
		gp.init(gp.getSize());
	}

	public static VinlGraphicPanel getGraphicPanel() {
		return gp;
	}

	private void drawCircles(int num) {
		System.out.println("Drawing circles starting: " + new java.util.Date().toString());
		int turn = 360 / num;
		for (int i = 0; i < num; i++) {
			for (int j = 0; j < 360; j++) {
				gp.forward(1);
				gp.right(1);
			}
			gp.left(turn);
		}
		gp.repaint();
		System.out.println("Drawing circles end: " + new java.util.Date().toString());
	}

	private void drawSquares(int iteration) {
		for (int i = 0; i < iteration; i++) {
			gp.forward(i + 2);
			gp.right(89);
		}
	}

	public void keyPressed(KeyEvent e) {
	}


	public void keyTyped(KeyEvent e) {
	}
	/*
	public void keyTyped(KeyEvent e) {
	char c = e.getKeyChar();
	switch (c) {
	case 'f':
	gp.forward(50);
	break;
	case 'b': 
	gp.back(50); 
	break;
	case 'c':
	gp.clearScreen();
	break;
	case 'q':
	System.exit(0);
	break;
	case 'd':
	drawSquares(40);
	break;
	case 'v':
	drawCircles(24);
	break;
	case 'p':
	gp.repaint();
	break;
	case 'r':
	gp.right(90);
	break;
	case 'l':
	gp.left(90);
	break;
	default:
	break;
	}
	e.consume();
	repaint();
	}
	 */

	public static void main(String args[]) {
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new VinlInterpreterGui().setVisible(true);
			}
		});
	}

	InputStream input;
	VinlLexer lexer;
	VinlParser parser;
	VinlAstWalker walker;
	
	public InputStream strToInputStream(String cmd) {
		InputStream in = null;
		if(cmd==null) return null;
		try {
			in = new ByteArrayInputStream(cmd.getBytes());
		} catch (Exception e) {}
		return in;
	}
	
	public void loadProgram(String filename) throws Exception {
		File src = new File(filename);
		FileInputStream fis = new FileInputStream(src);
		lexer = new VinlLexer(fis);
		parser = new VinlParser(lexer);
		parser.program();	
		walker = new VinlAstWalker();
		CommonAST tree = (CommonAST) parser.getAST();
		//System.out.println(tree.toStringList());
		VinlType r = walker.expr(tree);
	}

	public void keyReleased(KeyEvent e) {
		switch(e.getKeyCode()) {
			case KeyEvent.VK_ENTER:
				try {
					String cmd = editor.getText();
					cmdlist.add(cmd);
					cmd_index = cmdlist.size();
					if(cmd.startsWith("load ")) {
						loadProgram(cmd.substring(5).trim());
					} else {
						input = strToInputStream(cmd.trim());
						lexer = new VinlLexer(input);
						parser = new VinlParser(lexer);
						parser.cmd();
						walker = new VinlAstWalker();
						CommonAST tree = (CommonAST) parser.getAST();
						//System.out.println(tree.toStringList());
						VinlType r = walker.expr(tree);
					}
				} catch (Exception ex) {
					System.out.println(ex);
					VinlAstWalker.log(ex.getMessage(), VinlAstWalker.DEBUG_FINE);
					//ex.printStackTrace();
				}
				editor.setText("");
				break;
			case KeyEvent.VK_UP:
				if(cmd_index-1>=0) {
					editor.setText(cmdlist.get(--cmd_index));
				}
				break;
			case KeyEvent.VK_DOWN:
				if(cmd_index+1<cmdlist.size()) {
					editor.setText(cmdlist.get(++cmd_index));
				}
				break;
			default:
				break;
		}
		e.consume();
		repaint();
	}
}
