/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import antlr.RecognitionException;
import java.util.Hashtable;

/**
 * Vinl uses static scoping
 * @author ycao
 */
public class VinlSymbolTable {
	
	private Hashtable<String, VinlType> symbols = null;
	private VinlSymbolTable parent = null;
	
	public VinlSymbolTable() {
		symbols = new Hashtable();
	}

	public VinlSymbolTable(VinlSymbolTable parent) {
		symbols = new Hashtable();
		this.parent = parent;
	}

	public VinlSymbolTable getParent() {
		return parent;
	}

	public void setParent(VinlSymbolTable parent) {
		this.parent = parent;
	}
		
	public Hashtable getSymbols() {
		return this.symbols;
	}
	
	public void setSymbols(Hashtable symbols) {
		this.symbols = symbols;
	}
	
	public void put(String name, VinlType obj, boolean enable_redef) throws RecognitionException {
		//don't really expect this since parser should catch this error,
		//but better safe than sorry
		int type = obj.getType();
		if(type != VinlType.BOOLEAN && type != VinlType.VOID && 
		   		type != VinlType.DECIMAL && type != VinlType.FUNCTION) {
			throw new RecognitionException("ERROR: Undefined type.");
		}
		//Do not allow declare the same varible name twice unless override by enable_redef
		if(symbols.containsKey(name) && !enable_redef) {
			throw new RecognitionException("ERROR: Redefinition of variable: "+name);
		}
		symbols.put(name, obj);
	}

	public void put(String name, VinlType obj) throws RecognitionException { // this is used in tail recursion
		put(name, obj, false);
	}

	public VinlType get(String name) {
		VinlType ret = null;
		if(symbols.get(name) != null) {
			ret = symbols.get(name); 		
		} else {
			//check in parents' table
			VinlSymbolTable par = this.getParent();
			while(par != null) {
				if(par.getSymbols().get(name)!=null) {
					ret = (VinlType)(par.getSymbols().get(name));
					break;
				} else {
					par = par.getParent();
				}
			}
		}
		return ret;
	}

}
